'use client'

import Link from 'next/link'
import { usePathname } from 'next/navigation'
import { useState } from 'react'

import { AxiosError } from 'axios'
import Cookies from 'js-cookie'
import { BookHeart, KeyRound, ShoppingBag, UserCog } from 'lucide-react'

import { Button } from '@/components/ui/button'
import { useToast } from '@/components/ui/use-toast'
import useGetAccountData from '@/hooks/server/useGetAccountData'
import { cn } from '@/lib/utils'
import axiosClient from '@/services/axiosClient'
import { useUserStore } from '@/store/user'
import { deleteNextCookies } from '@/utils/setNextCookies'

const Navigation = () => {
  const pathName = usePathname()
  const { toast } = useToast()
  const setSkipAuthCheck = useUserStore((state) => state.setSkipAuthCheck)
  const [loading, setLoading] = useState(false)

  async function handleLogout() {
    setLoading(true)
    try {
      await axiosClient('sanctum/csrf-cookie')
      await axiosClient.post('api/account-logout')
      Cookies.remove('siamoods_token')
      deleteNextCookies('siamoods_token')
      setSkipAuthCheck(true)
      window.location.href = '/'
    } catch (error) {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    }
    setLoading(false)
  }

  const { accountData } = useGetAccountData()

  return (
    <div className='sticky top-28 w-max flex-shrink-0 rounded-lg border p-4 md:static md:w-full'>
      <h2 className='max-w-60 overflow-hidden text-ellipsis'>
        {accountData?.name} {accountData?.lastname}
      </h2>
      {/* <h3>{accountData?.email}</h3> */}
      <div className='divide-y'>
        <Link
          href={`/profile`}
          prefetch={false}
          className={cn(
            'mt-4 flex items-center gap-2 rounded-md px-4 py-2 pb-2 transition-colors hover:bg-muted md:pl-0',
            pathName === '/profile' && 'text-primary'
          )}
        >
          <UserCog size={22} strokeWidth={1} className=' translate-x-0.5' />
          Հաշիվ
        </Link>
        <Link
          href={`/orders`}
          prefetch={false}
          className={cn(
            'flex items-center gap-2 rounded-md px-4 py-2 transition-colors hover:bg-muted md:pl-0',
            pathName.startsWith('/orders') && 'text-primary'
          )}
        >
          <ShoppingBag size={22} strokeWidth={1} />
          Պատվերներ
        </Link>
        <Link
          href={`/favorites`}
          className={cn(
            'flex items-center gap-2 rounded-md px-4 py-2 transition-colors hover:bg-muted md:pl-0',
            pathName === '/favorites' && 'text-primary'
          )}
        >
          <BookHeart size={22} strokeWidth={1} />
          Նախընտրածներ
        </Link>
        <Link
          href={`/change-password`}
          className={cn(
            'flex items-center gap-2 rounded-md px-4 py-2 transition-colors hover:bg-muted md:pl-0',
            pathName === '/change-password' && 'text-primary'
          )}
        >
          <KeyRound size={22} strokeWidth={1} />
          Փոխել գաղտնաբառը
        </Link>
      </div>
      <Button
        variant='outline'
        className='mt-8'
        onClick={handleLogout}
        disabled={loading}
      >
        Դուրս գալ
      </Button>
    </div>
  )
}

export default Navigation
