'use client'

import Link from 'next/link'
import { useRef, useState } from 'react'

import { zodResolver } from '@hookform/resolvers/zod'
import { AxiosError } from 'axios'
import { motion } from 'framer-motion'
import Cookies from 'js-cookie'
import { useRouter } from 'next-nprogress-bar'
import ReCAPTCHA from 'react-google-recaptcha'
import { useForm } from 'react-hook-form'
import * as z from 'zod'

import SocialLogin from '@/components/SocialLogin'
import { Button, buttonVariants } from '@/components/ui/button'
import { Checkbox } from '@/components/ui/checkbox'
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form'
import { Input } from '@/components/ui/input'
import { Label } from '@/components/ui/label'
import { useToast } from '@/components/ui/use-toast'
import { cn } from '@/lib/utils'
import axiosClient from '@/services/axiosClient'

const formSchema = z
  .object({
    firstName: z.string().min(1, { message: 'Անունը պարտադիր է' }),
    lastName: z.string().min(1, { message: 'Ազգանունը պարտադիր է' }),
    email: z.string().min(1, { message: 'Էլ․ հասցեն պարտադիր է' }).email({
      message: 'Խնդրում ենք մուտքագրել վավեր էլ․ հասցե',
    }),
    password: z.string().min(1, { message: 'Գաղտնաբառը պարտադիր է' }),
    password_confirmation: z
      .string()
      .min(1, { message: 'Գաղտնաբառը պարտադիր է' }),
    subscribe: z.boolean(),
  })
  .refine((data) => data.password === data.password_confirmation, {
    message: 'Գաղտնաբառը չի համընկնում',
    path: ['password_confirmation'],
  })

const Register = () => {
  const { toast } = useToast()
  const router = useRouter()
  const [loading, setLoading] = useState(false)
  const recaptchaRef = useRef<ReCAPTCHA>(null)
  const [captchaToken, setCaptchaToken] = useState<string | null>(null)
  const form = useForm({
    resolver: zodResolver(formSchema),
    defaultValues: {
      firstName: '',
      lastName: '',
      email: '',
      password: '',
      password_confirmation: '',
      subscribe: false,
    },
  })

  async function handleRegister(values: z.infer<typeof formSchema>) {
    setLoading(true)
    try {
      await axiosClient('sanctum/csrf-cookie')
      const res = await axiosClient.post('api/account-register', {
        ...values,
        captchaToken,
      })
      toast({
        title: res.data.message,
      })
      Cookies.set('temporaryToken', res.data.temp_api_token)
      router.push('/login')
    } catch (error) {
      recaptchaRef.current?.reset()
      if (error instanceof AxiosError) {
        if (error?.response?.data.errors) {
          const message = Object.values(error.response.data.errors)[0] as string
          toast({
            title: message,
            variant: 'destructive',
          })
        }
      }
    }
    setLoading(false)
  }

  return (
    <motion.div
      initial={{ opacity: 0 }}
      animate={{ opacity: 1 }}
      exit={{ opacity: 0 }}
      transition={{
        type: 'spring',
        stiffness: 260,
        damping: 40,
      }}
      className='flex justify-center'
    >
      <div className='m-10 flex h-[680px] max-w-5xl overflow-hidden rounded-2xl shadow-2xl lg:m-4 lg:h-auto md:flex-col-reverse'>
        <div className='relative flex flex-1 flex-col items-center justify-center gap-8 p-8'>
          <div className='gradient' />
          <p className='text-center text-muted-foreground'>
            Մուտք գործեք հաշիվ և օգտագործեք այն՝ պատվերը ձևակերպելու համար
          </p>
          <Link
            href='/login'
            className={cn(
              buttonVariants({
                variant: 'outlineSecondary',
              })
            )}
          >
            Մուտք
          </Link>
          <SocialLogin borderPrimary={true} />
        </div>
        <Form {...form}>
          <form
            onSubmit={form.handleSubmit(handleRegister)}
            className='flex flex-[1.2] flex-col items-center justify-center gap-6 p-12 pt-6 lg:p-4'
          >
            <div className='flex flex-col items-center gap-2'>
              <h1>Ստեղծել հաշիվ</h1>
              <p className='text-muted-foreground'>
                Խնդրում ենք լրացնել դաշտերը
              </p>
            </div>
            <div className='flex w-full gap-4 lg:flex-col'>
              <FormField
                name='firstName'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel required={true}>Անուն</FormLabel>
                    <FormControl>
                      <Input {...field} />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <FormField
                name='lastName'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel required={true}>Ազգանուն</FormLabel>
                    <FormControl>
                      <Input {...field} />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
            </div>
            <FormField
              name='email'
              render={({ field }) => (
                <FormItem>
                  <FormLabel required={true}>էլ․ հասցե</FormLabel>
                  <FormControl>
                    <Input {...field} type='email' />
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />

            <div className='flex w-full gap-4 lg:flex-col'>
              <FormField
                name='password'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel required={true}>Գաղտնաբառ</FormLabel>
                    <FormControl>
                      <Input {...field} type='password' />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <FormField
                name='password_confirmation'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel required={true} className='whitespace-nowrap'>
                      Հաստատեք գաղտնաբառը
                    </FormLabel>
                    <FormControl>
                      <Input {...field} type='password' />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
            </div>
            <ReCAPTCHA
              ref={recaptchaRef}
              sitekey={process.env.NEXT_PUBLIC_RE_CAPTCHA_KEY as string}
              onChange={(token) => setCaptchaToken(token)}
            />

            <p className='text-center text-xs'>
              Բաժանորդագրվելով Դուք առաջինը կտեղեկանաք նոր ապրանքների և հատուկ
              առաջարկների մասին
            </p>
            <div className='flex items-center'>
              <FormField
                control={form.control}
                name='subscribe'
                render={({ field }) => (
                  <FormItem>
                    <FormControl>
                      <Checkbox
                        id='subscribe'
                        checked={field.value}
                        onCheckedChange={field.onChange}
                      />
                    </FormControl>
                  </FormItem>
                )}
              />
              <Label
                htmlFor='subscribe'
                className='pl-2 text-sm font-medium leading-none peer-disabled:cursor-not-allowed peer-disabled:opacity-70'
              >
                ԲԱԺԱՆՈՐԴԱԳՐՎԵԼ
              </Label>
            </div>
            <Button className='w-full font-bold' disabled={loading}>
              Ստեղծել հաշիվ
            </Button>
          </form>
        </Form>
      </div>
    </motion.div>
  )
}

export default Register
