'use client'

import { useRouter, useSearchParams } from 'next/navigation'
import { useState } from 'react'

import { zodResolver } from '@hookform/resolvers/zod'
import { AxiosError } from 'axios'
import { useForm } from 'react-hook-form'
import * as z from 'zod'

import { Button } from '@/components/ui/button'
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form'
import { Input } from '@/components/ui/input'
import { useToast } from '@/components/ui/use-toast'
import axiosClient from '@/services/axiosClient'

const formSchema = z
  .object({
    password: z.string().min(1, { message: 'Գաղտնաբառը պարտադիր է' }),
    password_confirmation: z
      .string()
      .min(1, { message: 'Գաղտնաբառը պարտադիր է' }),
  })
  .refine((data) => data.password === data.password_confirmation, {
    message: 'Գաղտնաբառը չի համընկնում',
    path: ['password_confirmation'],
  })

export default function ChangePassword() {
  const [loading, setLoading] = useState(false)
  const router = useRouter()
  const { toast } = useToast()
  const searchParams = useSearchParams()
  const email = searchParams.get('email') as string
  const token = searchParams.get('token') as string

  if (!email || !token) {
    router.push('/')
  }

  const form = useForm({
    resolver: zodResolver(formSchema),
    defaultValues: {
      password: '',
      password_confirmation: '',
    },
  })

  async function handlePasswordChange(values: z.infer<typeof formSchema>) {
    setLoading(true)
    try {
      await axiosClient.post('api/reset-password', {
        ...values,
        email,
        token,
        api_reset: 1,
      })
      toast({
        title: 'Գաղտնաբառը փոխված է',
      })
      router.push('/login')
    } catch (error) {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    }
    setLoading(false)
  }

  return (
    <div className='flex justify-center p-20'>
      <Form {...form}>
        <form
          onSubmit={form.handleSubmit(handlePasswordChange)}
          className='flex w-full max-w-lg flex-col items-center justify-center gap-6'
        >
          <h1 className='self-center text-2xl font-bold'>
            Վերականգնել գաղտնաբառը
          </h1>
          <FormField
            name='password'
            render={({ field }) => (
              <FormItem>
                <FormLabel required={true}>Նոր գաղտնաբառ</FormLabel>
                <FormControl>
                  <Input {...field} type='password' />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            name='password_confirmation'
            render={({ field }) => (
              <FormItem>
                <FormLabel required={true}>Հաստատեք գաղտնաբառը</FormLabel>
                <FormControl>
                  <Input {...field} type='password' />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          <Button className='w-full font-bold' disabled={loading}>
            Պահպանել
          </Button>
        </form>
      </Form>
    </div>
  )
}
