'use client'

import Link from 'next/link'
import { useState } from 'react'

import nProgress from 'nprogress'

import { cn, convertToMoney } from '@/lib/utils'
import type { ProductType } from '@/types'

import AddToFavoritesDialog from '../dialogs/AddToFavoritesDialog'
import AddedToCartDialog from '../dialogs/AddedToCartDialog'
import ViewProductDialog from '../dialogs/ViewProductDialog'
import ProductCardImages from './ProductCardImages'

const ProductCard = ({
  className,
  product,
  isInProduct = false,
  alwaysShowButtons,
}: {
  className?: string
  product: ProductType
  isInProduct?: boolean
  alwaysShowButtons?: boolean
}) => {
  // console.log('🚀 ~ file: ProductCard.tsx:22 ~ product:', product)
  const [selectedImage, setSelectedImage] = useState(0)
  const [currentAvailable, setCurrentAvailable] = useState(product.available)

  return (
    <div
      className={cn(
        'group isolate flex flex-col items-center overflow-hidden rounded-lg transition-shadow ',
        isInProduct ? 'hover:shadow-md' : 'hover:shadow-xl',
        className
      )}
      onMouseEnter={() => {
        if (product.images.length > 1) setSelectedImage(1)
      }}
      onMouseLeave={() => {
        if (product.images.length > 1) setSelectedImage(0)
      }}
    >
      <Link
        href={`/product/${product.slug}`}
        className='relative'
        data-prevent-nprogress={true}
        onClick={(e) => {
          const tagName = (e.target as Element).tagName.toLowerCase()
          if (tagName !== 'svg' && tagName !== 'button') {
            nProgress.start()
          }
        }}
      >
        {!!product.discount && (
          <div className='absolute left-4 top-4 z-10 rounded-sm bg-background px-2 py-1 text-sm font-bold text-red-600 shadow-sm'>
            -{product.discount}%
          </div>
        )}
        {!!product.label && (
          <div className='absolute right-0 top-4 z-10 min-w-16 rounded-s-full bg-background bg-teal-400 px-2 py-1 text-end text-sm font-bold text-white'>
            {product.label}
          </div>
        )}
        {!!product.discount_left && (
          <div className='absolute left-4 top-12 z-10 min-w-16 rounded-sm bg-background bg-red-100 px-2 py-1 text-end text-xs text-red-600'>
            {product.discount_left}
          </div>
        )}
        <ProductCardImages
          images={product.images}
          selectedImage={selectedImage}
          setSelectedImage={setSelectedImage}
          isInProduct={isInProduct}
        />
        {currentAvailable < 1 && (
          <div className='absolute bottom-0 left-1/2 -translate-x-1/2 text-sm'>
            ԱՌԿԱ ՉԷ
          </div>
        )}
      </Link>
      <div
        className={cn(
          'flex gap-2',
          !alwaysShowButtons &&
            'show-on-device-with-hover opacity-0 transition-opacity group-hover:opacity-100'
        )}
      >
        {!isInProduct && (
          <AddToFavoritesDialog
            haysellId={product.haysell_id}
            isFavoriteInitial={product.is_favorite}
            image={(product.images.length > 0 && product.images[0].image) || ''}
          />
        )}
        <div
          className={cn(
            currentAvailable < 1 && 'pointer-events-none opacity-0'
          )}
          onClick={() => {
            if (isInProduct) {
              setCurrentAvailable((prev) => prev - 1)
            }
          }}
        >
          <AddedToCartDialog
            product={product}
            quantity={1}
            haysellId={product.haysell_id}
          />
        </div>
        {!isInProduct && (
          <div className='hide-on-device-with-hover'>
            <ViewProductDialog
              slug={product.slug}
              haysellId={product.haysell_id}
            />
          </div>
        )}
      </div>
      <Link
        href={`/product/${product.slug}`}
        className={cn(
          'flex flex-col items-center justify-center',
          isInProduct ? 'p-4 pt-0' : 'p-6 pt-4'
        )}
      >
        <h3
          className={cn(
            'line-clamp-2 text-center',
            isInProduct ? 'mb-2 h-10 text-sm' : 'mb-6 h-12'
          )}
        >
          {product.title}
        </h3>
        {!!product.discount && (
          <h5 className='relative text-xs'>
            {convertToMoney(product.price)}
            <span className='absolute left-0 top-1/2 h-px w-full bg-red-500'></span>
          </h5>
        )}
        <h4
          className={cn(
            product.discount_price && 'text-red-600',
            isInProduct ? 'text-sm' : 'text-lg'
          )}
        >
          {product.discount_price
            ? convertToMoney(product.discount_price)
            : convertToMoney(product.price)}
        </h4>
        {isInProduct && (
          <div
            className={cn(
              'pt-2 text-sm font-bold',
              product.available > 0 ? 'text-green-500' : 'text-destructive'
            )}
          >
            {product.available > 0 ? 'Առկա է' : 'Առկա չէ'}
          </div>
        )}
      </Link>
    </div>
  )
}

export default ProductCard
