import Link from 'next/link'
import { useParams } from 'next/navigation'

import { useMutation, useQuery, useQueryClient } from '@tanstack/react-query'
import { AxiosError } from 'axios'
import { IoBagHandleOutline } from 'react-icons/io5'

import { Button } from '@/components/ui/button'
import {
  Dialog,
  DialogContent,
  DialogHeader,
  DialogTrigger,
} from '@/components/ui/dialog'
import { convertToMoney } from '@/lib/utils'
import { getCart } from '@/server/getCart'
import axiosClient from '@/services/axiosClient'
import { useUserStore } from '@/store/user'
import { ProductType } from '@/types'

import ImageWithFallback from '../ImageWithFallback'
import { useToast } from '../ui/use-toast'

const AddedToCartDialog = ({
  product,
  haysellId,
  isInProduct,
  quantity,
  variant,
}: {
  product?: ProductType
  haysellId?: number
  isInProduct?: boolean
  quantity: number | string
  variant?: string
}) => {
  const { toast } = useToast()
  const params = useParams()
  const slug = params.slug
  const queryClient = useQueryClient()
  const { data: cartData } = useQuery({
    queryKey: ['cart'],
    queryFn: getCart,
  })
  // const currentCartProductVariation = cartData?.products?.find(
  //   (item) => item.variation?.id === variant
  // )
  // console.log(
  //   '🚀 ~ file: AddedToCartDialog.tsx:50 ~ currentCartProductVariation:',
  //   currentCartProductVariation
  // )
  const itemsCount = cartData?.products?.reduce(
    (acc, item) => acc + item.quantity,
    0
  )
  const currentAddedProductDialog = useUserStore(
    (state) => state.currentAddedProductDialog
  )
  const setCurrentAddedProductDialog = useUserStore(
    (state) => state.setCurrentAddedProductDialog
  )
  const selectedViewProductDialog = useUserStore(
    (state) => state.selectedViewProductDialog
  )
  const setSelectedViewProductDialog = useUserStore(
    (state) => state.setSelectedViewProductDialog
  )
  const hasVariants =
    product?.variations && product.variations.length > 0 ? true : false
  // const available = variant
  //   ? product?.variations[+variant]?.available
  //   : product?.available
  // console.log(
  //   '🚀 ~ file: AddedToCartDialog.tsx:65 111111111~ variant:',
  //   variant
  // )
  // console.log(
  //   '🚀 ~ file: AddedToCartDialog.tsx:65 ~ available:',
  //   available,
  //   product
  // )

  const addToCartMutation = useMutation({
    mutationFn: () => {
      return axiosClient.post(`api/carts`, {
        products: [
          {
            haysell_id: haysellId,
            quantity,
            variation_id: variant,
          },
        ],
        is_cart: false,
      })
    },
    onSuccess: () => {
      if (haysellId) {
        setCurrentAddedProductDialog(haysellId)
      }
      queryClient.invalidateQueries({ queryKey: ['cart'] })
      // queryClient.invalidateQueries({ queryKey: ['product', product?.slug] })
      setSelectedViewProductDialog(undefined)
      // if (hasVariants && haysellId) {
      //   setCurrentAddedProductDialog(haysellId)
      // }
    },
    onError: (error) => {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    },
  })

  return (
    <Dialog
      open={currentAddedProductDialog === haysellId}
      onOpenChange={(open) => {
        if (!open) setCurrentAddedProductDialog(undefined)
      }}
    >
      <DialogTrigger asChild>
        {isInProduct ? (
          <Button
            onClick={() => {
              if (
                hasVariants &&
                haysellId &&
                !selectedViewProductDialog &&
                !slug
              ) {
                setSelectedViewProductDialog(haysellId)
              } else {
                addToCartMutation.mutate()
              }
            }}
            disabled={addToCartMutation.isPending}
          >
            ԱՎԵԼԱՑՆԵԼ ԶԱՄԲՅՈՒՂ
          </Button>
        ) : (
          <Button
            variant='ghost'
            size='iconSm'
            onClick={() => {
              if (hasVariants && haysellId) {
                setSelectedViewProductDialog(haysellId)
              } else {
                addToCartMutation.mutate()
              }
            }}
            disabled={addToCartMutation.isPending}
          >
            <i className='text-xl'>
              <IoBagHandleOutline className='[&>path]:stroke-[40]' />
            </i>
          </Button>
        )}
      </DialogTrigger>
      <DialogContent className='gap-0 p-0'>
        <DialogHeader className='bg-muted p-4 text-start text-muted-foreground'>
          <h2>Ապրանքն ավելացվել է Ձեր զամբյուղում</h2>
        </DialogHeader>
        <div className='flex gap-4 p-8 md:p-4 sm:flex-col sm:items-center'>
          <Link href='/'>
            <ImageWithFallback
              src={
                (product?.images &&
                  product?.images?.length > 0 &&
                  product?.images[0].image) ||
                ''
              }
              alt='product'
              width={80}
              height={80}
              className='aspect-square rounded-md object-cover'
            />
          </Link>
          <div className='flex flex-1 flex-col items-start gap-1 sm:items-center'>
            <Link
              href='/'
              className='line-clamp-1 text-base transition-colors hover:text-primary'
            >
              {product?.title}
            </Link>
            {variant && <div>Չափս: {variant}</div>}
          </div>
          <div>
            {quantity} x {convertToMoney(product?.price)}
          </div>
        </div>
        <div className='space-y-4 border-t p-4'>
          <div className='flex justify-between'>
            <div className='text-sm'>Զամբյուղում կա {itemsCount} ապրանք</div>
            <div className='whitespace-nowrap pl-1 sm:text-sm'>
              Ընդհանուր:{' '}
              <span className='font-bold'>{convertToMoney(cartData?.sum)}</span>
            </div>
          </div>
          <div className='flex justify-between pr-4'>
            <DialogTrigger asChild className='mt-0'>
              <Button variant='ghost'>Շարունակել գնումները</Button>
            </DialogTrigger>
            <DialogTrigger asChild>
              <Link href='/checkout'>
                <Button>Պատվիրել</Button>
              </Link>
            </DialogTrigger>
          </div>
        </div>
      </DialogContent>
    </Dialog>
  )
}

export default AddedToCartDialog
