'use client'

import * as SliderPrimitive from '@radix-ui/react-slider'
import React, { useEffect, useState } from 'react'

import { cn, convertToMoney } from '@/lib/utils'
import { usePathname, useSearchParams } from 'next/navigation'
import { useRouter } from 'next-nprogress-bar'

type SliderProps = {
  className?: string
  min: number
  max: number
  step: number
  value?: [number, number]
  onValueChange?: (values: [number, number]) => void
}

const SliderRange = React.forwardRef(
  (
    { className, min, max, step, value, onValueChange, ...props }: SliderProps,
    ref
  ) => {
    const searchParams = useSearchParams()
    const minParam = searchParams.get('min_price')
    const maxParam = searchParams.get('max_price')
    const minValue = searchParams.get('min_price') ?? min
    const maxValue = searchParams.get('max_price') ?? max
    const pathname = usePathname()
    const router = useRouter()
    const initialValue = Array.isArray(value) ? value : [+minValue, +maxValue]
    const [localValues, setLocalValues] = useState(initialValue)

    const handleValueChange = (newValues: [number, number]) => {
      setLocalValues(newValues)
      if (onValueChange) {
        onValueChange(newValues)
      }
    }

    useEffect(() => {
      if (!minParam || !maxParam) {
        setLocalValues([min, max])
      }
    }, [minParam, maxParam])

    useEffect(() => {
      setLocalValues([+minValue, +maxValue])
    }, [min, max])

    const handlePointerUp = () => {
      const params = new URLSearchParams(window.location.search)
      params.set('min_price', localValues[0].toString())
      params.set('max_price', localValues[1].toString())
      router.push(`${pathname}?${params.toString()}`)
    }

    return (
      <SliderPrimitive.Root
        ref={ref as React.RefObject<HTMLDivElement>}
        min={min}
        max={max}
        step={step}
        value={localValues}
        onValueChange={handleValueChange}
        onPointerUp={handlePointerUp}
        className={cn(
          'relative mb-6 mt-14 flex w-full touch-none select-none items-center',
          className
        )}
        {...props}
      >
        <SliderPrimitive.Track className='relative h-1.5 w-full grow overflow-hidden rounded-full bg-primary/20'>
          <SliderPrimitive.Range className='absolute h-full bg-primary' />
        </SliderPrimitive.Track>
        {localValues.map((value, index) => (
          <React.Fragment key={index}>
            <div
              className={cn(
                'absolute -top-10 h-16 w-1/2',
                index === 0 ? 'left-0' : 'right-0 text-end'
              )}
            >
              <span>{convertToMoney(value)}</span>
            </div>
            <div
              className={cn(
                'absolute -bottom-6 text-center text-xs text-muted-foreground',
                index === 0 ? 'left-0' : 'right-0'
              )}
            >
              <span>{convertToMoney(index === 0 ? min : max)}</span>
            </div>
            <SliderPrimitive.Thumb className='block h-4 w-4 rounded-full border border-primary/50 bg-background shadow transition-colors focus-visible:outline-none focus-visible:ring-1 focus-visible:ring-ring disabled:pointer-events-none disabled:opacity-50' />
          </React.Fragment>
        ))}
      </SliderPrimitive.Root>
    )
  }
)

SliderRange.displayName = SliderPrimitive.Root.displayName

export { SliderRange }
