import { useEffect, useRef, useState } from 'react'

import { keepPreviousData, useQuery } from '@tanstack/react-query'

import { useToast } from '@/components/ui/use-toast'
import { getCheckout } from '@/server/getCheckout'
import { useCheckoutStore } from '@/store/checkout'
import type { Checkout } from '@/types'

import useGetShippingData from './useGetShippingData'

const useGetCheckoutData = () => {
  const [checkoutValidData, setCheckoutValidData] = useState<Checkout>()
  const deletedItemIds = useRef<Set<number>>(new Set())

  const { shippingData } = useGetShippingData()
  const setCheckoutStore = useCheckoutStore((state) => state.setCheckoutStore)
  const deliveryTypeId = useCheckoutStore((state) => state.deliveryTypeId)
  const promotion_code = useCheckoutStore((state) => state.promotion_code)
  const gift_card = useCheckoutStore((state) => state.gift_card)
  const promotion = useCheckoutStore((state) => state.promotion)
  const email = useCheckoutStore((state) => state.email)
  const { toast } = useToast()

  const handleFromDeleteCheckoutValidData = (id: number) => {
    deletedItemIds.current.add(id)
    setCheckoutValidData((prev) => {
      if (!prev) return
      return {
        ...prev,
        products: prev.products.filter((item) => item.id !== id),
      }
    })
  }

  const currentShipping = shippingData?.find(
    (item) => item.shipping_type_id === +deliveryTypeId
  )

  const {
    data: checkoutData,
    isFetching,
    isFetched,
    error,
    isError,
  } = useQuery({
    queryKey: [
      'checkout',
      {
        shipping_area_id: currentShipping?.shipping_area_id,
        promotion_code,
        email,
        gift_card_id: gift_card,
        promotion_id: promotion,
      },
    ],
    queryFn: () =>
      getCheckout({
        shipping_area_id: currentShipping?.shipping_area_id,
        promotion_code: promotion_code || undefined,
        email: email || undefined,
        gift_card_id: gift_card || undefined,
        promotion_id: promotion || undefined,
      }),
    enabled: Boolean(currentShipping?.shipping_area_id),
    placeholderData: keepPreviousData,
    staleTime: 0,
  })

  useEffect(() => {
    if (isFetched && checkoutData && !isError) {
      const filteredData = {
        ...checkoutData,
        products: checkoutData.products.filter(
          (item) =>
            item.id !== undefined && !deletedItemIds.current.has(item.id)
        ),
      }

      setCheckoutValidData(filteredData)
      setCheckoutStore('promotion_code', '')
      setCheckoutStore('promotionCodeValue', '')
      if (checkoutData.gift_card?.id) {
        setCheckoutStore('gift_card', checkoutData.gift_card.id.toString())
      }
      if (checkoutData.promotion?.id) {
        setCheckoutStore('promotion', checkoutData.promotion.id.toString())
      }
    }
  }, [isFetched, checkoutData, isError])

  useEffect(() => {
    if (isError) {
      toast({
        title: error.toString(),
        variant: 'destructive',
      })
      setCheckoutStore('promotion_code', '')
      setCheckoutStore('promotionCodeValue', '')
    }
  }, [isError])

  return {
    checkoutData: checkoutValidData,
    handleFromDeleteCheckoutValidData,
    isFetching,
  }
}

export default useGetCheckoutData
